require "watchman"
require "logger"
require "logdna"

require "mtr_monitor/version"
require "mtr_monitor/hop"
require "mtr_monitor/metrics"
require "mtr_monitor/log_dna"
require "mtr_monitor/cleaner"

module MtrMonitor

  #
  # MtrMonitor generates a MTR report for a domain and saves the report to:
  # /var/log/mtr/<name>__<YYYY-DD-MM-HH-MM>__<host-ip-address>_to_<target-ip-address>.txt
  #
  # After that, it uploads the report to S3:
  # s3://<bucket-name>/<name>/<YYYY-DD-MM-HH-MM>/<host-ip-address>_to_<target-ip-address>.txt
  #
  # Parameters:
  #
  # Name:        The name of this report.    Example: "hetzner-to-us-east-1"
  # Domain:      The domain to hit with MTR. Example: "github.com"
  # MTR Options: Options to pass to the mtr command line tool.
  #

  LOCAL_REPORTS_DIR = "/var/log/mtr"

  class Report
    attr_reader :log_path
    attr_reader :s3_path
    attr_reader :logger
    attr_reader :host_ip_address

    def initialize(options)
      @name                  = options.fetch(:name)
      @domain                = options.fetch(:mtr_target)
      @s3_bucket             = options.fetch(:s3_bucket)
      @aws_access_key_id     = options.fetch(:aws_access_key_id)
      @aws_secret_access_key = options.fetch(:aws_secret_access_key)
      @mtr_options           = options.fetch(:mtr_options)
      @dig_ip_address        = options.fetch(:dig_ip_address)
      @logger                = options.fetch(:logger, Logger.new(STDOUT))
      @logdna_ingestion_key  = options.fetch(:logdna_ingestion_key)

      @host_ip_address       = `dig +short myip.opendns.com @resolver1.opendns.com`.strip
      @hostname              = `hostname`.strip
      @created_at            = Time.now.utc
    end

    def s3_url
      "https://s3.eu-central-1.amazonaws.com/#{@s3_bucket}/#{path}"
    end

    def s3_uri
      "s3://#{@s3_bucket}/#{path}"
    end

    def local_log_path
      "#{MtrMonitor::LOCAL_REPORTS_DIR}/#{path.gsub("/", "__")}"
    end

    def path
      "#{@name}/#{@created_at.strftime("%Y-%m-%d_%H-%M")}/#{@host_ip_address.gsub(".", "-")}_to_#{@domain.gsub(".", "-")}.txt"
    end

    def generate
      logger.info "Local Path: #{local_log_path}"
      logger.info "S3 URI: #{s3_uri}"

      generate_local_report
      upload_to_s3

      MtrMonitor::LogDna.submit(@name, @logdna_ingestion_key, local_log_path, @host_ip_address, s3_url, @hostname, @logger)
      MtrMonitor::Metrics.submit(local_log_path, @hostname, @name, @logger)
    end

    def generate_local_report
      run %Q(sudo mkdir -p /var/log/mtr)
      run %Q(sudo sh -c 'echo "Version:   #{MtrMonitor::VERSION.ljust(30)}" >> #{local_log_path}')
      run %Q(sudo sh -c 'echo "Name:      #{@name.ljust(30)}" >> #{local_log_path}')
      run %Q(sudo sh -c 'echo "Domain:    #{@domain.ljust(30)}" >> #{local_log_path}')
      run %Q(sudo sh -c 'echo "Source IP: #{@host_ip_address.ljust(30)}" >> #{local_log_path}')
      run %Q(sudo sh -c 'echo "Target IP: #{destination.ljust(30)}" >> #{local_log_path}')
      run %Q(sudo sh -c 'echo "-----------------------------------------" >> #{local_log_path}')
      run %Q(sudo sh -c 'mtr --report --report-wide #{@mtr_options} #{destination} >> #{local_log_path}')
    end

    def destination
      @dig_ip_address ? dig : @domain
    end

    def upload_to_s3
      run %Q(AWS_ACCESS_KEY_ID='#{@aws_access_key_id}' AWS_SECRET_ACCESS_KEY='#{@aws_secret_access_key}' aws s3 cp #{local_log_path} #{s3_uri})
    end

    def dig
      logger.info "Dig #{@domain} IP address"
      run("dig +short #{@domain}").split("\n").first
    end

    def run(cmd)
      logger.info "Running: #{cmd}"

      `#{cmd}`
    end
  end
end
